//
//  ResourceManager.h
//  GameEngine
//
//  Created by Gary MacDonald on 6/22/2012.
//  Copyright 2012 midgetdivet. All rights reserved.
//

#import <OpenGLES/EAGL.h>
#import <OpenGLES/EAGLDrawable.h>
#import <OpenGLES/ES1/gl.h>
#import <OpenGLES/ES1/glext.h>
#import <Foundation/Foundation.h>
#import "GLTexture.h"
#import "GLFont.h"
@class GLESGameState;
@class ResourceManager;

#define STORAGE_FILENAME @"appstorage"

// Some helper methods.  These don't really belong in this class.
// Returns the distance between two points.
CGFloat distsquared(CGPoint a, CGPoint b);
// Returns a unit vector pointing from a to b.
CGPoint toward(CGPoint a, CGPoint b);

extern ResourceManager* g_ResManager; //paul <3's camel caps, hungarian notation, and underscores.

@interface ResourceManager : NSObject {
	// Used to allocate and manage GLTexture instances.  Needs to be cleared in dealloc.
	NSMutableDictionary*    textures;
	// Used to track sound allocations.  The actual sound data is buffered in SoundEngine; 'sounds' here only tracks the openAL ids of the loaded sounds.
	NSMutableDictionary*    sounds;
	NSMutableDictionary*    storage;
	BOOL                    storage_dirty;
	NSString*               storage_path;
	GLFont*                 default_font;
}

+(ResourceManager*)instance;
-(void)shutdown;
// Loads and buffers images as 2d opengles textures.
-(GLTexture*)getTexture:(NSString*)filename;
-(void)purgeTextures;
// Intialize the sound device.  Takes a non-trivial amount of time, and should be called during initialization.
-(void)setupSound;
// Useful for preloading sounds; called automatically by playSound.  Buffers sounds.
-(UInt32)getSound:(NSString*)filename;
-(void)purgeSounds;
// Play a sound.  Loads and buffers the sound if needed.
-(void)playSound:(NSString*)filename;
// Play and loop a music file in the background.  Streams the file.
-(void)playMusic:(NSString*)filename;
// Stop the music.  Unloads the currently playing music file.
-(void)stopMusic;
// Useful for loading binary files that you include in the program bundle, such as game level data
-(NSData*) getBundleData:(NSString*)filename;
// For saving preferences or other game data.  This is stored in the documents directory, and may persist between app version updates.
-(BOOL) storeUserData:(id)data toFile:(NSString*)filename;
// For loading prefs or other data saved with storeData.  Returns nil if the file does not exist.
-(id) getUserData:(NSString*)filename;
-(BOOL) userDataExists:(NSString*)filename;
+(NSString*) appendStorePath:(NSString*)filename;
-(GLFont*) defaultFont;
-(void) setDefaultFont:(GLFont*)newValue;

@end
